"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Job = void 0;
/* eslint-disable no-unused-vars */
const eventemitter3_1 = require("eventemitter3");
const lodash_1 = require("lodash");
class Job extends eventemitter3_1.EventEmitter {
    /**
       * Cancel Job
       * @example
       * ```js
       * xdccJS.on('ready', async () => {
       *  const job1 = xdccJS.download('bot', '20-30')
       *  job1.on('downloaded', info => {
       *    // stop half-way
       *    if(info.success.length === 5) job1.cancel()
       *  })
       * })
       * ```
       */
    cancel;
    /** @ignore */
    failures;
    /** @ignore */
    nick;
    /** @ignore */
    cancelNick;
    /** @ignore */
    now;
    /** @ignore */
    queue;
    /** @ignore */
    retry;
    /** @ignore */
    success;
    /** @ignore */
    timeout;
    /** @ignore */
    opts;
    constructor(candidate, cancelFn) {
        // eslint-disable-next-line constructor-super
        super();
        this.cancel = () => {
            cancelFn();
            this.emit('cancel', candidate, 'cancelled by client');
        };
        this.failures = candidate.failures;
        this.nick = candidate.nick;
        this.cancelNick = candidate.cancelNick || candidate.nick;
        this.now = candidate.now;
        this.queue = candidate.queue;
        this.retry = candidate.retry;
        this.success = candidate.success;
        this.timeout = candidate.timeout;
        this.opts = candidate.opts;
    }
    /**
       * return current job
       * @example
       * ```js
       * const job = await xdccJS.download('bot', '1-5')
       * console.log(job.show().nick) //=> 'bot'
       * console.log(job.show().queue) //=> [2, 3, 4, 5]
       * console.log(job.show().now) //=> 1
       * // etc...
       *
       * // both achieve same results:
       * job.cancel()
       * job.show().cancel()
       * ```
       */
    show() {
        const info = {
            nick: (0, lodash_1.clone)(this.nick),
            queue: (0, lodash_1.clone)(this.queue),
            now: (0, lodash_1.clone)(this.now),
            success: (0, lodash_1.clone)(this.success),
            failed: (0, lodash_1.clone)(this.failures),
            opts: (0, lodash_1.clone)(this.opts),
            cancel: this.cancel,
        };
        return info;
    }
    /**
       * check if Job is done
       * const job = await xdccJS.download('bot', '1-5')
       * job.isDone() //=> false
       */
    isDone() {
        if (this.queue.length) {
            return false;
        }
        return true;
    }
}
exports.Job = Job;
